<?php
/*
   TM Offline Stats v1.1
   Copyright (c) 2006-2009 Alexander Domnin

   This program is licenced under the
   Creative Commons Attribution-Noncommercial-No Derivative Works 3.0 Unported.
   To view a copy of this licence, visit http://creativecommons.org/licenses/by-nc-nd/3.0/

   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
   without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*/

require_once "tmos_funcs.inc.php";

// *****************************************************************************
// Ttmos_DB
// *****************************************************************************

class Ttmos_DB {
   var $cfg;
   var $funcs;
   var $db_link;
   var $safe;
   var $last_error;

   function Ttmos_DB($config, $safefl = false) {
      $this->cfg = $config;
      if ($this->cfg['dbtype'] != "mysql" &&
          $this->cfg['dbtype'] != "mssql" &&
          $this->cfg['dbtype'] != "pg")
         $this->cfg['dbtype'] = "mysql";
      $this->funcs['open'] = "open_".$this->cfg['dbtype'];
      $this->funcs['close'] = "close_".$this->cfg['dbtype'];
      $this->funcs['select'] = "query_select_".$this->cfg['dbtype'];
      $this->funcs['selectpage'] = "query_selectpage_".$this->cfg['dbtype'];
      $this->funcs['exec'] = "query_exec_".$this->cfg['dbtype'];
      $this->funcs['schema'] = "instance_schema_".$this->cfg['dbtype'];
      $this->funcs['tables_drop'] = "tables_drop_".$this->cfg['dbtype'];
      $this->funcs['tables_create'] = "tables_create_".$this->cfg['dbtype'];
      $this->funcs['version'] = "db_version_".$this->cfg['dbtype'];
      $this->db_link = NULL;
      $this->safe = $safefl;
      $this->last_error = '';
   }

   function last_error() {
      return $this->last_error;
   }
   
// MySQL (sm +) ////////////////////////////////////////////////////////////////

   function open_mysql() {
      $this->db_link = @mysql_connect($this->cfg['dbhost'], $this->cfg['dblogin'],
                                         $this->cfg['dbpassword']);
      if (!$this->db_link ||
          !@mysql_select_db($this->cfg['dbname'], $this->db_link) ||
          !@mysql_query("/*!40101 SET NAMES 'utf8' */", $this->db_link)) {
         if ($this->safe) $this->last_error = mysql_error();
         else die("Ttmos_DB: ".mysql_error());
         if ($this->db_link !== false)
            @mysql_close($this->db_link);
         if ($this->last_error == '') $this->last_error = '?';
      }
      else
         $this->last_error = '';
   }

   function close_mysql() {
      if ($this->db_link != null && $this->db_link !== false) {
         if (!@mysql_close($this->db_link)) {
            if ($this->safe) $this->last_error = mysql_error();
            else die("Ttmos_DB: ".mysql_error());
         }
         else
            $this->last_error = '';
         $this->db_link = null;
      }
   }

   function query_select_mysql($sql, $key1, $key2, $defval = null) {
      $query_result = @mysql_unbuffered_query($sql, $this->db_link);
      if ($query_result === false) {
         if ($this->safe) $this->last_error = mysql_error($this->db_link);
         else die("Ttmos_DB: ".mysql_error($this->db_link)." (".$sql.")");
         if ($key1 != -1 || $key2 != -1)
            $result = array();
         else
            $result = array("count"=>0);
      }
      else {
         $this->last_error = '';
         
         $result = array();
         if ($key2 != -1) {
            while ($curr_str = mysql_fetch_array($query_result, MYSQL_ASSOC)) {
               $result[$curr_str[$key1]][$curr_str[$key2]] = $curr_str;
               unset($result[$curr_str[$key1]][$curr_str[$key2]][$key2]);
               unset($result[$curr_str[$key1]][$curr_str[$key2]][$key1]);
            }
         }
         else if ($key1 != -1) {
            while ($curr_str = mysql_fetch_array($query_result, MYSQL_ASSOC)) {
               $result[$curr_str[$key1]] = $curr_str;
               unset($result[$curr_str[$key1]][$key1]);
            }
         }
         else {
            $result['count'] = 0;
            while ($curr_str = mysql_fetch_array($query_result, MYSQL_ASSOC)) {
               if ($defval !== null) {
                  foreach ($curr_str as $key=>$value)
                     if ($value == null) $curr_str[$key] = $defval;
               }
               $result[$result['count']] = $curr_str;
               $result['count'] += 1;
            }
         }
         mysql_free_result($query_result);
      }
      return $result;
   }

   function query_exec_mysql($sql) {
      //echo $sql."\r\n";
      $result = @mysql_query($sql, $this->db_link);
      if ($result === false) {
         if ($this->safe) $this->last_error = mysql_error($this->db_link);
         else die("Ttmos_DB: ".mysql_error($this->db_link)." (".$sql.")");
      }
      else
         $this->last_error = '';
   }

   function query_selectpage_mysql($sql, $pagenum, $recperpage) {
      $sql .= " LIMIT ".($pagenum-1)*$recperpage.", $recperpage";
      return $this->query_select_mysql($sql, -1, -1);
   }

   function db_version_mysql() {
      $result = @mysql_get_server_info($this->db_link);
      if ($result === false) {
         if ($this->safe) $this->last_error = mysql_error($this->db_link);
         else die("Ttmos_DB: ".mysql_error($this->db_link));
         $result = '?';
      }
      else {
         $this->last_error = '';
         preg_match('/[1-9]{1,3}\.[0-9]{1,3}.[0-9]{0,4}/', $result, $buf);
         isset($buf[0]) ? $result = $buf[0] : $result = '?';
      }
      return $result;
   }

   // sm -
   // update for PHP 5.3+
   function instance_schema_mysql() {
      $result = array();
      $tbls = $this->query_select_mysql(
         "show tables from {$this->cfg['dbname']} like 'tmos_%'", -1, -1);
      for ($i = 0; $i < $tbls['count']; $i++) {
         foreach ($tbls[$i] as $tablename) {
            $flds = $this->query_select_mysql(
               "show columns from {$tablename}", -1, -1);
            for ($j = 0; $j < $flds['count']; $j++)
               $result[$tablename][$flds[$j]['Field']] = "1";
         }
      }

      return $result;
   }

// MsSQL (sm +) ////////////////////////////////////////////////////////////////

   function open_mssql() {
      if (preg_match("/(.+):(\d{3,7})$/i", $this->cfg['dbhost'], $buf))
         $this->cfg['dbhost'] = $buf[1].",".$buf[2];
      $this->db_link = @mssql_connect($this->cfg['dbhost'], $this->cfg['dblogin'],
                                      $this->cfg['dbpassword']);
      if (!$this->db_link ||
          !@mssql_select_db($this->cfg['dbname'], $this->db_link)) {
         if ($this->safe) $this->last_error =
            "Connection failed ({$this->cfg['dbhost']}, {$this->cfg['dblogin']})";
         else die("Ttmos_DB: Connection failed");
         if ($this->db_link !== false)
            @mssql_close($this->db_link);
      }
      else
         $this->last_error = '';
   }

   function close_mssql() {
      if ($this->db_link != null && $this->db_link !== false) {
         if (!@mssql_close($this->db_link)) {
            if ($this->safe) $this->last_error = mssql_get_last_message();
            else die("Ttmos_DB: ".mssql_get_last_message());
         }
         else
            $this->last_error = '';
         $this->db_link = null;
      }
   }

   function query_select_mssql($sql, $key1, $key2, $defval = null) {
      // prepare qry (instr -> charindex)
      //$sql = preg_replace("/instr[ ]*\(([^,]+),([^\)]+)\)/i", 'charindex(\\2,\\1)', $sql);

      $query_result = @mssql_query($sql, $this->db_link);
      if ($query_result === false) {
         if ($this->safe) $this->last_error = mssql_get_last_message();
         else die("Ttmos_DB: ".mssql_get_last_message()." (".$sql.")");
         if ($key1 != -1 || $key2 != -1)
            $result = array();
         else
            $result = array("count"=>0);
      }
      else {
         $this->last_error = '';
         
         $result = array();
         if ($key2 != -1) {
            while ($curr_str = mssql_fetch_array($query_result, MSSQL_ASSOC)) {
               $result[$curr_str[$key1]][$curr_str[$key2]] = $curr_str;
               unset($result[$curr_str[$key1]][$curr_str[$key2]][$key2]);
               unset($result[$curr_str[$key1]][$curr_str[$key2]][$key1]);
            }
         }
         else if ($key1 != -1) {
            while ($curr_str = mssql_fetch_array($query_result, MSSQL_ASSOC)) {
               $result[$curr_str[$key1]] = $curr_str;
               unset($result[$curr_str[$key1]][$key1]);
            }
         }
         else {
            $result['count'] = 0;
            while ($curr_str = mssql_fetch_array($query_result, MSSQL_ASSOC)) {
               if ($defval !== null) {
                  foreach ($curr_str as $key=>$value)
                     if ($value == null) $curr_str[$key] = $defval;
               }
               $result[$result['count']] = $curr_str;
               $result['count'] += 1;
            }
         }
         mssql_free_result($query_result);
      }

      return $result;
   }

   function query_exec_mssql($sql) {
      $result = @mssql_query($sql, $this->db_link);
      if ($result === false) {
         if ($this->safe) $this->last_error = mssql_get_last_message();
         else die("Ttmos_DB: ".mssql_get_last_message()." (".$sql.")");
      }
      else
         $this->last_error = '';
   }

   function query_selectpage_mssql($sql, $pagenum, $recperpage) {
      // ,    order by,
      //    select    ,
      //    , ,   tmos ... 
      $buf = "select count(*) as tr from (".substr($sql, 0, strpos($sql, 'order by')-1).") t";
      $buf = $this->query_select_mssql($buf, -1, -1);
      if ($recperpage >= $buf[0]['tr'])
         $buf = 0;
      else if ($pagenum*$recperpage > $buf[0]['tr'])
         $buf = $buf[0]['tr'] - ($pagenum-1)*$recperpage;
      else
         $buf = $recperpage;
      if ($buf != 0) {
         $orderbyf = preg_replace("/ .{1}\./i", " ", substr($sql, strpos($sql, 'order by')));
         $orderbyb = str_replace(array(" asc", " desc", " !desc"),
                                 array(" !desc", " asc", " desc"), $orderbyf);
         $sql = "select * from (".
                   "select top $buf * from (".
                      preg_replace("/^select (.+)/i", "select top ".
                                    ($pagenum*$recperpage)." \\1", $sql).") t1 ".
                   "$orderbyb) t2 ".
                "$orderbyf";
      }
      return $this->query_select_mssql($sql, -1, -1);
   }

   function db_version_mssql() {
      $result = $this->query_select_mssql('select @@version as ver', -1, -1);
      if ($result['count'] > 0) {
         preg_match('/[1-9]{1,3}\.[0-9]{1,3}.[0-9]{0,4}/', $result[0]['ver'], $buf);
         isset($buf[0]) ? $result = trim($buf[0]) : $result = '?';
      }
      else
         $result = '?';
      return $result;
   }

   function instance_schema_mssql() {
      $result = array();
      $tbls = $this->query_select_mssql(
         "select name, id from dbo.sysobjects where name like 'tmos_%'", -1, -1);
      for ($i = 0; $i < $tbls['count']; $i++) {
         $flds = $this->query_select_mssql(
            "select name from dbo.syscolumns where id = {$tbls[$i]['id']}", -1, -1);
         for ($j = 0; $j < $flds['count']; $j++)
            $result[$tbls[$i]['name']][$flds[$j]['name']] = "1";
      }
      return $result;
   }

// PostgreSQL (sm +) ///////////////////////////////////////////////////////////

   function open_pg() {
      $cs = "dbname='{$this->cfg['dbname']}' ".
            "user='{$this->cfg['dblogin']}' ".
            "password='{$this->cfg['dbpassword']}' ".
            "connect_timeout=5";
      if (preg_match("/(.+):(\d{3,7})$/i", $this->cfg['dbhost'], $buf))
         $cs = "host='{$buf[1]}' port={$buf[2]} ".$cs;
      else
         $cs = "host='{$this->cfg['dbhost']}' port=5432 ".$cs;
      $this->db_link = @pg_connect($cs);
      if (!$this->db_link || 
          @pg_set_client_encoding($this->db_link, 'UTF8') == -1) {
         if ($this->safe) $this->last_error = "Connection failed ($cs)";
         else die("Ttmos_DB: Connection failed");
         if ($this->db_link !== false)
            @pg_close($this->db_link);
      }
      else
         $this->last_error = '';
   }

   function close_pg() {
      if ($this->db_link != null && $this->db_link !== false) {
         if (!@pg_close($this->db_link)) {
            if ($this->safe) $this->last_error = pg_last_error();
            else die("Ttmos_DB: ".pg_last_error());
         }
         else
            $this->last_error = '';
         $this->db_link = null;
      }
   }

   function query_select_pg($sql, $key1, $key2, $defval = null) {
      //$sql = preg_replace("/instr[ ]*\(([^,]+),([^\)]+)\)/i", 'position(\\2 in \\1)', $sql);

      $query_result = @pg_query($this->db_link, $sql);
      if ($query_result === false) {
         if ($this->safe) $this->last_error = pg_last_error($this->db_link);
         else die("Ttmos_DB: ".pg_last_error($this->db_link)." (".$sql.")");
         if ($key1 != -1 || $key2 != -1)
            $result = array();
         else
            $result = array("count"=>0);
      }
      else {
         $this->last_error = '';
         
         $result = array();
         if ($key2 != -1) {
            while ($curr_str = pg_fetch_assoc($query_result)) {
               $result[$curr_str[$key1]][$curr_str[$key2]] = $curr_str;
               unset($result[$curr_str[$key1]][$curr_str[$key2]][$key2]);
               unset($result[$curr_str[$key1]][$curr_str[$key2]][$key1]);
            }
         }
         else if ($key1 != -1) {
            while ($curr_str = pg_fetch_assoc($query_result)) {
               $result[$curr_str[$key1]] = $curr_str;
               unset($result[$curr_str[$key1]][$key1]);
            }
         }
         else {
            $result['count'] = 0;
            while ($curr_str = pg_fetch_assoc($query_result)) {
               if ($defval !== null) {
                  foreach ($curr_str as $key=>$value)
                     if ($value == null) $curr_str[$key] = $defval;
               }
               $result[$result['count']] = $curr_str;
               $result['count'] += 1;
            }
         }
         pg_free_result($query_result);
      }
      return $result;
   }

   function query_exec_pg($sql) {
      $result = @pg_query($this->db_link, $sql);
      if ($result === false) {
         if ($this->safe) $this->last_error = pg_last_error($this->db_link);
         else die("Ttmos_DB: ".pg_last_error($this->db_link)." (".$sql.")");
      }
      else
         $this->last_error = '';
   }

   function query_selectpage_pg($sql, $pagenum, $recperpage) {
      $sql .= " LIMIT $recperpage OFFSET ".($pagenum-1)*$recperpage;
      return $this->query_select_pg($sql, -1, -1);
   }

   function db_version_pg() {
      $result = $this->query_select_pg('select version() as ver', -1, -1);
      if ($result['count'] > 0) {
         preg_match('/[1-9]{1,3}\.[0-9]{1,3}.[0-9]{0,4}/', $result[0]['ver'], $buf);
         isset($buf[0]) ? $result = trim($buf[0]) : $result = '?';
      }
      else
         $result = '?';
      return $result;
   }

   function instance_schema_pg() {
      $result = array();
      $tbls = $this->query_select_pg(
         "select table_name from information_schema.tables ".
         "where table_name like 'tmos_%'", -1, -1);
      for ($i = 0; $i < $tbls['count']; $i++) {
         $flds = $this->query_select_pg(
            "select column_name from information_schema.columns ".
            "where table_name = '{$tbls[$i]['table_name']}'", -1, -1);
         for ($j = 0; $j < $flds['count']; $j++)
            $result[$tbls[$i]['table_name']][$flds[$j]['column_name']] = "1";
      }
      return $result;
   }

// Overall (sm -) //////////////////////////////////////////////////////////////
   function instance_schemas($ver = '') {
      $result = array();
      $result['v1.00'] = array(
         "tmos_servers" => array("serverid"=>1, "server"=>1, "serverc"=>1, "ip"=>1, "game"=>1,
                                 "login"=>1, "password"=>1, "trackdirs"=>1, "logfiles"=>1,
                                 "envirs"=>1, "lastupdate"=>1),
         "tmos_players" => array("serverid"=>1, "playerid"=>1, "account"=>1, "player"=>1,
                                 "playerc"=>1, "links"=>"1", "aliases"=>1, "lastonline"=>1),
         "tmos_tracks"  => array("serverid"=>1, "trackid"=>1, "uid"=>1, "track"=>1, "trackc"=>1,
                                 "author"=>1, "envirid"=>1, "mood"=>1, "type"=>1, "nblaps"=>1,
                                 "price"=>1, "bronze"=>1, "silver"=>1, "gold"=>1, "authortime"=>1,
                                 "authorscore"=>1, "rc"=>1, "pc"=>1, "filename"=>1),
         "tmos_results" => array("serverid"=>1, "trackid"=>1, "playerid"=>1, "result"=>1, "ts"=>1,
                                 "fp"=>"1", "medal"=>"1"),
         "tmos_clans"   => array("serverid"=>1, "clanid"=>1, "clanc"=>1, "description"=>1,
                                 "members"=>1, "mc"=>1),
         "tmos_scores"  => array("serverid"=>1, "ownerid"=>1, "envirid"=>1, "am"=>1, "gm"=>1,
                                 "sm"=>1, "bm"=>1, "fc"=>1, "score"=>1, "rank"=>1)
      );
      $result['v0.62'] = array(
         "tmos_servers" => array("serverid"=>1, "server"=>1, "serverc"=>1, "ip"=>1, "game"=>1,
                                 "login"=>1, "password"=>1, "tracks"=>"1", "logfiles"=>"1",
                                 "last_update"=>"1"),
         "tmos_players" => array("serverid"=>1, "playerid"=>1, "account"=>1, "player"=>1,
                                 "playerc"=>1, "aliases"=>1, "lastonline"=>1, "am"=>1, "gm"=>1,
                                 "sm"=>1, "bm"=>1, "score"=>1, "am_c"=>1, "gm_c"=>1, "sm_c"=>1,
                                 "bm_c"=>1, "score_c"=>1),
         "tmos_tracks"  => array("serverid"=>1, "trackid"=>1, "uid"=>1, "track"=>1, "trackc"=>1,
                                 "author"=>1, "envir"=>1, "mood"=>1, "type"=>1, "nblaps"=>1,
                                 "price"=>1, "bronze"=>1, "silver"=>1, "gold"=>1, "authortime"=>1,
                                 "authorscore"=>1, "relpath"=>1),
         "tmos_results" => array("serverid"=>1, "resultid"=>1, "trackid"=>1, "playerid"=>1,
                                 "result"=>1, "ts"=>1),
         "tmos_clans"   => array("serverid"=>1, "clanid"=>1, "clanc"=>1, "description"=>1,
                                 "members"=>1, "score"=>1, "score_c"=>1, "tm"=>1),
         "tmos_view"    => array("serverid"=>1, "stats"=>1, "trackid"=>1, "playerid"=>1,
                                 "bestresult"=>1, "ts"=>1, "award"=>1, "rank"=>1, "totalresults"=>1)
      );
      $result['v0.61'] = array(
         "tmos_servers" => array("serverid"=>1, "server"=>1, "serverc"=>1, "ip"=>1, "game"=>1,
                                 "login"=>1, "password"=>1, "tracks"=>1, "logfiles"=>1,
                                 "last_update"=>1),
         "tmos_players" => array("serverid"=>1, "playerid"=>1, "account"=>1, "player"=>1,
                                 "playerc"=>1, "aliases"=>1, "lastonline"=>1, "am"=>1, "gm"=>1,
                                 "sm"=>1, "bm"=>1, "score"=>1, "am_c"=>1, "gm_c"=>1, "sm_c"=>1,
                                 "bm_c"=>1, "score_c"=>1),
         "tmos_tracks"  => array("serverid"=>1, "trackid"=>1, "uid"=>1, "track"=>1, "trackc"=>1,
                                 "author"=>1, "envir"=>1, "mood"=>1, "type"=>1, "nblaps"=>1,
                                 "price"=>1, "bronze"=>1, "silver"=>1, "gold"=>1, "authortime"=>1,
                                 "authorscore"=>1, "relpath"=>1),
         "tmos_results" => array("serverid"=>1, "resultid"=>1, "trackid"=>1, "playerid"=>1,
                                 "result"=>1, "ts"=>1),
         "tmos_clans"   => array("serverid"=>1, "clanid"=>1, "clanc"=>1, "members"=>1, "score"=>1,
                                 "score_c"=>1, "tm"=>1),
         "tmos_view"    => array("serverid"=>1, "stats"=>1, "trackid"=>1, "playerid"=>1,
                                 "bestresult"=>1, "ts"=>1, "award"=>1, "rank"=>1, "totalresults"=>1)
      );
      $result['v0.60'] = array(
         "tmos_servers" => array("serverid"=>1, "server"=>1, "serverc"=>1, "ip"=>1, "game"=>1,
                                 "login"=>1, "password"=>1, "tracks"=>1, "logfiles"=>1,
                                 "last_update"=>1),
         "tmos_players" => array("serverid"=>1, "playerid"=>1, "account"=>1, "player"=>1,
                                 "playerc"=>1, "aliases"=>1, "lastonline"=>1, "am"=>1, "gm"=>1,
                                 "sm"=>1, "bm"=>1, "score"=>1, "am_c"=>1, "gm_c"=>1, "sm_c"=>1,
                                 "bm_c"=>1, "score_c"=>1),
         "tmos_tracks"  => array("serverid"=>1, "trackid"=>1, "uid"=>1, "track"=>1, "trackc"=>1,
                                 "author"=>1, "envir"=>1, "mood"=>1, "type"=>1, "nblaps"=>1,
                                 "price"=>1, "bronze"=>1, "silver"=>1, "gold"=>1, "authortime"=>1,
                                 "authorscore"=>1, "relpath"=>1),
         "tmos_results" => array("serverid"=>1, "resultid"=>1, "trackid"=>1, "playerid"=>1,
                                 "result"=>1, "ts"=>1),
         "tmos_clans"   => array("serverid"=>1, "clanid"=>1, "clanc"=>1, "members"=>1, "score"=>1,
                                 "score_c"=>1, "tm"=>1),
         "tmos_view"    => array("serverid"=>1, "stats"=>1, "trackid"=>1, "playerid"=>1,
                                 "bestresult"=>1, "ts"=>1, "award"=>1, "rank"=>1)
      );
      $result['v0.51'] = array(
         "tmos_servers" => array("serverid"=>1, "server"=>1, "serverc"=>1, "ip"=>1, "game"=>1,
                                 "login"=>1, "password"=>1, "tracksdir"=>1, "logfile"=>1,
                                 "parse_position"=>1, "parse_track"=>1, "parse_str"=>1,
                                 "parse_last"=>1),
         "tmos_players" => array("serverid"=>1, "playerid"=>1, "account"=>1, "ip"=>1, "player"=>1,
                                 "playerc"=>1, "nation"=>"1", "aliases"=>1, "lastonline"=>1,
                                 "am"=>1, "gm"=>1, "sm"=>1, "bm"=>1, "score"=>1, "rank"=>1),
         "tmos_tracks"  => array("serverid"=>1, "trackid"=>1, "uid"=>1, "track"=>1, "trackc"=>1,
                                 "author"=>1, "envir"=>1, "mood"=>1, "type"=>1, "nblaps"=>1,
                                 "price"=>1, "bronze"=>1, "silver"=>1, "gold"=>1, "authortime"=>1,
                                 "authorscore"=>1, "filename"=>1),
         "tmos_results" => array("serverid"=>1, "trackid"=>1, "playerid"=>1, "besttime"=>1,
                                 "ts"=>1, "award"=>1),
      );
      $result['v0.50'] = array(
         "tmos_servers" => array("serverid"=>1, "server"=>1, "serverc"=>1, "ip"=>1, "game"=>1,
                                 "login"=>1, "password"=>1, "tracksdir"=>1, "logfile"=>1,
                                 "parse_position"=>1, "parse_track"=>1, "parse_str"=>1,
                                 "parse_last"=>1),
         "tmos_players" => array("serverid"=>1, "playerid"=>1, "player"=>1, "playerc"=>1,
                                 "account"=>1, "ip"=>1, "nation"=>"1", "am"=>1, "gm"=>1, "sm"=>1,
                                 "bm"=>1, "score"=>1, "rank"=>1),
         "tmos_tracks"  => array("serverid"=>1, "trackid"=>1, "uid"=>1, "track"=>1, "trackc"=>1,
                                 "author"=>1, "envir"=>1, "mood"=>1, "type"=>1, "nblaps"=>1,
                                 "price"=>1, "bronze"=>1, "silver"=>1, "gold"=>1, "authortime"=>1,
                                 "authorscore"=>1, "filename"=>1),
         "tmos_results" => array("serverid"=>1, "trackid"=>1, "playerid"=>1, "besttime"=>1,
                                 "ts"=>1, "award"=>1),
      );
      $result['v0.30/v0.40/v0.41/v0.42/v0.43/v0.44'] = array(
         "tmos_players" => array("id"=>1, "player"=>1, "ip"=>1, "am"=>1, "gm"=>1, "sm"=>1,
                                 "bm"=>1, "score"=>1),
         "tmos_tracks"  => array("id"=>1, "uid"=>1, "name"=>1, "author"=>1, "envir"=>1,
                                 "mood"=>1, "type"=>1, "nblaps"=>1, "price"=>1, "bronze"=>1,
                                 "silver"=>1, "gold"=>1, "authortime"=>1, "authorscore"=>1,
                                 "filename"=>1),
         "tmos_results" => array("trackid"=>1, "playerid"=>1, "besttime"=>1, "ts"=>1,
                                 "prize"=>1),
      );
      if ($ver != '' && isset($result[$ver]))
         return $result[$ver];
      else
         return $result;
   }
   
   function instance_schema_status() {
      $schemas = $this->instance_schemas();
      $schema_inst = $this->{$this->funcs['schema']}();

      // test v1.00
      $result['count'] = 0;
      foreach ($schemas['v1.00'] as $curr_tbl=>$flds) {
         $result[$result['count']]['table'] = $curr_tbl;
         $result[$result['count']]['status'] = "1";
         $result[$result['count']]['record_count'] = "?";
         if (!isset($schema_inst[$curr_tbl]))
            $result[$result['count']]['status'] = "0";
         else {
            foreach ($flds as $curr_fld=>$tmp) {
               if (!isset($schema_inst[$curr_tbl][$curr_fld])) {
                  $result[$result['count']]['status'] = "-1";
                  break;
               }
            }
         }
         if ($result[$result['count']]['status'] == "1") {
            $buf = $this->{$this->funcs['select']}("select count(*) as rc from ".$curr_tbl, -1, -1);
            $result[$result['count']]['record_count'] = $buf[0]['rc'];
         }
         $result['count'] += 1;
      }
      $result['ver'] = "v1.00";
      $result['update'] = '0';
      $result['support'] = '0';
      for ($i = 0; $i < $result['count']; $i++) {
         if ($result[$i]['status'] != "1") {
            $result['ver'] = "?";
            break;
         }
      }
      // test v0.30-v0.62
      if ($result['ver'] == "?") {
         foreach ($schemas as $curr_ver=>$tbls) {
            $result['ver'] = $curr_ver;
            foreach ($tbls as $curr_tbl=>$flds) {
               if (!isset($schema_inst[$curr_tbl])) {
                  $result['ver'] = "?";
                  break;
               }
               else
                  foreach ($flds as $curr_fld=>$tmp)
                     if (!isset($schema_inst[$curr_tbl][$curr_fld])) {
                        $result['ver'] = "?";
                        break;
                     }
            }
            if ($result['ver'] == $curr_ver)
               break;
         }
         if ($result['ver'] == "?") {
            $result['update'] = '0';
            $result['support'] = '0';
         }
         else {
            $result['update'] = '1';
            if ($result['ver'] == 'v0.30/v0.40/v0.41/v0.42/v0.43/v0.44' ||
                $result['ver'] == 'v0.50' ||
                $result['ver'] == 'v0.51')
               $result['support'] = '0';
            else
               $result['support'] = '1';
         }
      }
      // last version
      $result['ver_last'] = "v1.00";

      return $result;
   }

   function open() {
      switch ($this->cfg['dbtype']) {
      case 'mysql':
         if (!function_exists('mysql_connect'))
            $this->last_error = 'MySQL not supported by PHP';
         break;
      case 'mssql':
         if (!function_exists('mssql_connect'))
            $this->last_error = 'MsSQL not supported by PHP';
         break;
      case 'oracle':
         if (!function_exists('oci_connect'))
            $this->last_error = 'Oracle not supported by PHP';
         break;
      case 'pg':
         if (!function_exists('pg_connect'))
            $this->last_error = 'PostgreSQL not supported by PHP';
         break;
      }
      if ($this->last_error == '') {
         $this->{$this->funcs['open']}();
      }
   }

   function close() {
      $this->{$this->funcs['close']}();
   }
   
   function plug_lu(&$view) {
      $sql = "select lastupdate from tmos_servers where serverid = {$view['sid']}";
      $data = $this->{$this->funcs['select']}($sql, -1, -1);

      return $data;
   }
   
   function plug_top10(&$view) {
      $sql = "select s.rank, s.score, p.player from tmos_players p, tmos_scores s ".
             "where s.serverid = {$view['sid']} and s.envirid = '{$view['eid']}' ".
               "and p.serverid = s.serverid and p.playerid = s.ownerid ".
             "order by s.score desc";
      $data = $this->{$this->funcs['selectpage']}($sql, $view['pagenum'], $view['recperpage']);
      $sql = "select s.rank, s.score, p.player from tmos_players p, tmos_scores s ".
             "where s.serverid = {$view['sid']} and s.ownerid = '{$view['pid']}' ".
               "and s.envirid = '{$view['eid']}' ".
               "and p.serverid = s.serverid and p.playerid = s.ownerid";
      $data['player'] = $this->{$this->funcs['select']}($sql, -1, -1);

      return $data;
   }

   function plug_recs(&$view) {
      $sql = "select r.fp, r.result, p.player from tmos_results r, tmos_players p ".
             "where r.serverid = {$view['sid']} and r.trackid = '{$view['tid']}' ".
               "and p.serverid = r.serverid and p.playerid = r.playerid ".
             "order by result";
      $data = $this->{$this->funcs['selectpage']}($sql, $view['pagenum'], $view['recperpage']);
      $sql = "select r.fp, r.result, p.player from tmos_results r, tmos_players p ".
             "where r.serverid = {$view['sid']} and r.trackid = '{$view['tid']}' ".
               "and r.playerid = '{$view['pid']}' ".
               "and p.serverid = r.serverid and p.playerid = r.playerid";
      $data['player'] = $this->{$this->funcs['select']}($sql, -1, -1);

      return $data;
   }

   function plug_tt(&$view) {
      $sql = "select t.track, t.bronze, t.silver, t.gold, t.authortime ".
             "from tmos_tracks t ".
             "where t.serverid = {$view['sid']} and t.trackid = '{$view['tid']}'";
      $data = $this->{$this->funcs['select']}($sql, -1, -1);
      $sql = "select r.result, p.player from tmos_results r, tmos_players p ".
             "where r.serverid = {$view['sid']} and r.trackid = '{$view['tid']}' ".
               "and r.playerid = '{$view['pid']}' ".
               "and p.serverid = r.serverid and p.playerid = r.playerid";
      $data['player'] = $this->{$this->funcs['select']}($sql, -1, -1);

      return $data;
   }

   function plug_teams(&$view) {
      $sql = "select s.rank, s.score, c.mc, c.clanc ".
             "from tmos_scores s, tmos_clans c ".
             "where s.serverid = {$view['sid']} and s.envirid = '{$view['eid']}' ".
               "and c.serverid = s.serverid and c.clanid = s.ownerid ".
             "order by score desc";
      $data = $this->{$this->funcs['selectpage']}($sql, $view['pagenum'], $view['recperpage']);

      return $data;
   }

   function plug_stats(&$view) {
      $sql = "select p.player, s.am, s.gm, s.sm, s.bm, s.fc, s.score, s.rank ".
             "from tmos_scores s, tmos_players p ".
             "where s.serverid = {$view['sid']} and s.ownerid = '{$view['pid']}' ".
               "and s.envirid = '{$view['eid']}' and p.serverid = s.serverid ".
               "and p.playerid = s.ownerid";
      $data = $this->{$this->funcs['select']}($sql, -1, -1);
      if ($data['count'] > 0) {
         $data[0]['ft'] = $data[0]['am'] + $data[0]['gm'] +
                          $data[0]['sm'] + $data[0]['bm'] +
                          $data[0]['fc'];
         $sql = "select count(r.trackid) as fp from tmos_results r, tmos_tracks t ".
                "where r.serverid = {$view['sid']} and r.playerid = '{$view['pid']}' ".
                  ($view['eid'] == '*' ? "" : " and t.envirid = '{$view['eid']}' ").
                  "and r.serverid = t.serverid and r.trackid = t.trackid and r.fp = 1";
         $data[0]['fp'] = $this->{$this->funcs['select']}($sql, -1, -1);
         $data[0]['fp'] = $data[0]['fp'][0]['fp'];
         $sql = "select sum(r.fp) as afp from tmos_results r, tmos_tracks t ".
                "where r.serverid = {$view['sid']} and r.playerid = '{$view['pid']}' ".
                  ($view['eid'] == '*' ? "" : " and t.envirid = '{$view['eid']}'").
                  "and r.serverid = t.serverid and r.trackid = t.trackid";
         $data[0]['afp'] = $this->{$this->funcs['select']}($sql, -1, -1, 0);
         $data[0]['afp'] = $data[0]['afp'][0]['afp']/$data[0]['ft'];
      }
      
      return $data;
   }

   function plug_srv(&$view) {
      $sql = "select server from tmos_servers where serverid = {$view['sid']}";
      $data = $this->{$this->funcs['select']}($sql, -1, -1);
      $sql = "select count(p.playerid) as tp, coalesce(sum(s.am+s.gm+s.sm+s.bm), 0) as tm ".
             "from tmos_players p, tmos_scores s ".
             "where p.serverid = {$view['sid']} and s.envirid = '{$view['eid']}' ".
               "and p.serverid = s.serverid and p.playerid = s.ownerid";
      $buf = $this->{$this->funcs['select']}($sql, -1, -1);
      $data[0]['tp'] = $buf[0]['tp'];
      $data[0]['tm'] = $buf[0]['tm'];
      $sql = "select count(distinct t.trackid) as tt ".
             "from tmos_tracks t, tmos_results r ".
             "where t.serverid = {$view['sid']} and t.serverid = r.serverid ".
               "and t.trackid = r.trackid".
               ($view['eid'] == '*' ? "" : " and t.envirid = '{$view['eid']}'");
      $buf = $this->{$this->funcs['select']}($sql, -1, -1);
      $data[0]['tt'] = $buf[0]['tt'];
      $sql = "select count(c.clanid) as tc from tmos_clans c, tmos_scores s ".
             "where c.serverid = '{$view['sid']}' and c.serverid = s.serverid ".
               "and c.clanid = s.ownerid and s.envirid = '{$view['eid']}'";
      $buf = $this->{$this->funcs['select']}($sql, -1, -1);
      $data[0]['tc'] = $buf[0]['tc'];
      
      return $data;
   }

} // class