<?php
/*
   TMOS plugin for (X)Aseco/FAST v5
   Copyright (c) 2006-2009 Alexander Domnin

   This program is licenced under the
   Creative Commons Attribution-Noncommercial-No Derivative Works 3.0 Unported.
   To view a copy of this licence, visit http://creativecommons.org/licenses/by-nc-nd/3.0/

   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
   without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*/

require_once "tmos_funcs.inc.php";
require_once "tmos_db.inc.php";

// *****************************************************************************
// Ttmos_Plug
// *****************************************************************************

class Ttmos_Plug {
   var $cfg;
   var $db;
   var $status;

   function init($filename) {
      $this->status = array("cfg"=>0, "db"=>0, "schema"=>0, "srv"=>0, "plug"=>"d");
      // cfg
      $this->cfg = $this->config_read($filename);
      if ($this->cfg != -1)
         $this->status['cfg'] = 1;
      else
         $this->status['cfg'] = -1;
      // db
      if ($this->status['cfg'] == 1) {
         $this->db = new Ttmos_DB($this->cfg, true);
         $this->db->open();
         if ($this->db->last_error == '')
            $this->status['db'] = 1;
         else
            $this->status['db'] = -1;
      }
      // schema
      if ($this->status['db'] == 1) {
         $this->status['schema'] = $this->db->instance_schema_status();
         if ($this->status['schema']['ver'] ==
             $this->status['schema']['ver_last'])
            $this->status['schema'] = 1;
         else
            $this->status['schema'] = -1;
      }
      // srv
      if ($this->status['schema'] == 1) {
         $buf = array("sid"=>$this->cfg['dbserverid']);
         $this->status['srv'] = $this->db->plug_lu($buf);
         if ($this->status['srv']['count'] > 0)
            $this->status['srv'] = 1;
         else
            $this->status['srv'] = -1;
      }
      // plug
      if ($this->status['srv'] == 1)
         $this->status['plug'] = "e";
      else
         $this->status['plug'] = "d";

      $checks = array(1=>"Success", 0=>"Not Tested", -1=>"Fail",
                      "e"=>"Enabled", "d"=>"Disabled");
      $result = array("count"=>7,
                      0=>"[TMOS] TM Offline Stats Plugin v4 ...",
                      1=>"[TMOS] (c) 2006-2009, Alexander Domnin (http://tmos.pp.ru)",
                      2=>"[TMOS] Load settings file ($filename): ".$checks[$this->status['cfg']],
                      3=>"[TMOS] Try to connect to DB server: ".$checks[$this->status['db']],
                      4=>"[TMOS] Check DB schema version (need v1.0): ".$checks[$this->status['schema']],
                      5=>"[TMOS] Search for server ID: ".$checks[$this->status['srv']],
                      6=>"[TMOS] Plugin: ".$checks[$this->status['plug']]);

      return $result;
   }

   function config_read($filename) {
      $result = array();
      $vals = array();
      $tags = array();
      $it_ids  = array("csm", "dsm", "mlp");
      $it_strs = array("lu"=>1, "tt"=>1, "tt_nf"=>1, "tt_author"=>2, "tt_gold"=>2,
                       "tt_silver"=>2, "tt_bronze"=>2, "tt_player"=>1, "tt_player_str"=>2,
                       "tt_player_nf"=>1, "t10"=>1, "t10_nf"=>1, "t10_str"=>3, "t10_player"=>1,
                       "t10_player_nf"=>1, "stats"=>1, "stats_nf"=>1, "stats_rank"=>2,
                       "stats_score"=>2, "stats_medals"=>2, "stats_ft"=>2, "stats_fp"=>2,
                       "stats_afp"=>2, "recs"=>1, "recs_nf"=>1, "recs_str"=>3, "recs_player"=>1,
                       "recs_player_nf"=>1, "teams"=>1, "teams_nf"=>1, "teams_str"=>4,
                       "srv"=>1, "srv_tp"=>2, "srv_tt"=>2, "srv_ta"=>2, "srv_tc"=>2, "help"=>1,
                       "help_c"=>1, "help_tt"=>2, "help_top10"=>2, "help_stats"=>2, "help_recs"=>2,
                       "help_teams"=>2, "help_srv"=>2, "help_time"=>2, "time"=>1, "unknown"=>1
                       );

      if ($result != -1) {
         if (!file_exists($filename))
            $result = -1;
      }

      if ($result != -1) {
         $buf = file_get_contents($filename);
         if (!$buf)
            $result = -1;
      }

      if ($result != -1) {
         $xml_parser = xml_parser_create('UTF-8');
         xml_parser_set_option($xml_parser, XML_OPTION_SKIP_WHITE, 0);
         $buf = xml_parse_into_struct($xml_parser, $buf, $vals, $tags);
         xml_parser_free($xml_parser);
         if (!$buf)
            $result = -1;
      }

      if ($result != -1 &&
          isset($tags['DBHOST'][0]) &&
          isset($tags['DBTYPE'][0]) &&
          isset($tags['DBNAME'][0]) &&
          isset($tags['DBLOGIN'][0]) &&
          isset($tags['DBPASSWORD'][0]) &&
          isset($tags['DBSERVERID'][0]) &&
          isset($tags['WRITE_TO_CHAT'][0])) {
         $result['dbhost']         = $vals[$tags['DBHOST'][0]]['value'];
         $result['dbtype']         = $vals[$tags['DBTYPE'][0]]['value'];
         $result['dbname']         = $vals[$tags['DBNAME'][0]]['value'];
         $result['dblogin']        = $vals[$tags['DBLOGIN'][0]]['value'];
         $result['dbpassword']     = $vals[$tags['DBPASSWORD'][0]]['value'];
         $result['dbserverid']     = $vals[$tags['DBSERVERID'][0]]['value'];
         $result['write_to_chat']  = (strtoupper($vals[$tags['WRITE_TO_CHAT'][0]]['value'])
                                      == "TRUE" ? true : false);
      }
      else {
         $result = -1;
      }

      if ($result != -1 &&
          isset($tags['MLP.WINDOW'][0])) {
         $buf = $vals[$tags['MLP.WINDOW'][0]]['value'];
         $buf = preg_split("/,/", $buf);
         if (count($buf) == 7) {
            $result['mlp.posx'] = $buf[0];
            $result['mlp.posy'] = $buf[1];
            $result['mlp.width'] = $buf[2];
            $result['mlp.height1'] = $buf[3];
            $result['mlp.height2'] = $buf[4];
            $result['mlp.mlid'] = $buf[5];
            $result['mlp.aid'] = $buf[6];
         }
         else
            $result = -1;
      }

      if ($result != -1) {
         foreach ($it_ids as $id) {
            foreach ($it_strs as $str=>$cc)
               if (!isset($tags[strtoupper("$id.$str")][0]))
                  $result = -1;
         }
      }

      if ($result != -1) {
         foreach ($it_ids as $id) {
            foreach ($it_strs as $str=>$cc) {
               if (isset($tags[strtoupper("$id.$str.cfg")][0]))
                  $result["$id.$str"] = $this->cols_read(
                     $id, $vals[$tags[strtoupper("$id.$str")][0]]['value'],
                     $vals[$tags[strtoupper("$id.$str.cfg")][0]]['value'], $cc);
               else 
                  $result["$id.$str"] = $this->cols_read(
                     $id, $vals[$tags[strtoupper("$id.$str")][0]]['value'], -1, $cc);
            }
         }
      }

      return $result;
   }

   function cols_read($sendto, $it, $cf, $cc) {
      $result = array("count"=>$cc);
      $it = preg_split("/;/", $it);
      $cf = preg_split("/,/", $cf);
      if ($sendto == "mlp") {
         for ($i = 0; $i < $cc; $i++) {
            $result[$i] = array("text"=>"", "width"=>-1,
                                "align"=>-1, "bgclr"=>-1);
            if (isset($it[$i]))     $result[$i]['text']  = $it[$i];
            if (isset($cf[3*$i]))   $result[$i]['width'] = $cf[3*$i];
            if (isset($cf[3*$i+1])) $result[$i]['align'] = $cf[3*$i+1];
            if (isset($cf[3*$i+1])) $result[$i]['bgclr'] = $cf[3*$i+2];
         }
      }
      else {
         for ($i = 0; $i < $cc; $i++) {
            $result[$i] = array("text"=>"", "align"=>-1);
            if (isset($it[$i])) $result[$i]['text']  = $it[$i];
            if (isset($cf[$i])) $result[$i]['align'] = $cf[$i];
         }
      }

      return $result;
   }

   function cols_align($value, $maxlen, $align) {
      $result = $value;
      $padlen = $maxlen - strlen($result);
      for ($i = 0; $i < $padlen; $i++)
         if ($align == 'left')
            $result = $result.'  ';
         else if ($align == 'right')
            $result = '  '.$result;
         else
            $result = ' '.$result.' ';
      return $result;
   }

   function cols_format($sendto, $it, $args = null) {
      // format
      foreach ($args as $an=>$data) {
         for ($sn = 0; $sn < $data['count']; $sn++) {
            foreach ($data[$sn] as $column=>$value) {
               if ($column == "score" || $column == "afp")
                  $args[$an][$sn][$column] = float_format($value);
               if (strpos("authortime,gold,silver,bronze,result,", "$column,") !== false &&
                   substr($value, 0, 3) == "00:")
                  $args[$an][$sn][$column] = substr($value, 3);
               if ($column == "lastupdate")
                  $args[$an][$sn][$column] = ts_format($value);
            }
         }
      }
      if ($sendto != "csm" && $sendto != "dsm")
         return;
      // calc width
      $cw = array();
      for ($cn = 0; $cn < $it['count']; $cn ++) {
         $cw[$cn] = 0;
      }
      foreach ($args as $an=>$data) {
         for ($sn = 0; $sn < $data['count']; $sn++) {
            $cn = 0;
            foreach ($data[$sn] as $column=>$value) {
               if ($it[$cn]['align'] != -1 &&
                   $cw[$cn] < strlen($value))
                  $cw[$cn] = strlen($value);
               $cn ++;
            }
         }
      }
      // set width
      foreach ($args as $an=>$data) {
         for ($sn = 0; $sn < $data['count']; $sn++) {
            $cn = 0;
            foreach ($data[$sn] as $column=>$value) {
               $args[$an][$sn][$column] = $this->cols_align($value, $cw[$cn], $it[$cn]['align']);
               $cn ++;
            }
         }
      }

   }

   function msg_str($sendto, $it) {
      // data
      $args = func_get_args();
      $sendto = array_shift($args);
      $it = array_shift($args);
      $ss = array('{1}', '{2}', '{3}', '{4}');
      // csm, dsm
      if ($sendto == "csm" || $sendto == "dsm") {
         $result = "";
         for ($i = 0; $i < $it['count']; $i++)
            $result .= str_replace($ss, $args, $it[$i]['text']);
         $result .= "\$z\n";
      }
      // mlp
      else {
         $result = "<line>";
         for ($i = 0; $i < $it['count']; $i++) {
            $result .= '<cell'.
                       ($it[$i]['width'] != -1 ? ' width="'.$it[$i]['width'].'"' : "").
                       ($it[$i]['bgclr'] != -1 ? ' bgcolor="'.$it[$i]['bgclr'].'"' : "").'><text'.
                       ($it[$i]['align'] != -1 ? ' halign="'.$it[$i]['align'].'"' : "").'>'.
                       htmlspecialchars(str_replace($ss, $args, $it[$i]['text'])).'</text></cell>';
         }
         $result .= '</line>';
      }

      return $result;
   }

   function msg_help($view) {
      $msg   = $this->msg_str($view['st'], $this->cfg[$view['st'].'.help_c']);
      $msg  .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.help_tt']);
      $msg  .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.help_top10']);
      $msg  .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.help_stats']);
      $msg  .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.help_recs']);
      $msg  .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.help_teams']);
      $msg  .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.help_srv']);
      $msg  .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.help_time']);

      return $msg;
   }

   function msg_top10($view) {
      // data
      $data = $this->db->plug_top10($view);
      // format
      $this->cols_format($view['st'], $this->cfg[$view['st'].'.t10_str'],
                         array(&$data, &$data['player']));
      // msg
      if ($data['count'] == 0)
         $msg = $this->msg_str($view['st'], $this->cfg[$view['st'].'.t10_nf']);
      else {
         $msg = $this->msg_str($view['st'], $this->cfg[$view['st'].'.t10']);
         for ($i = 0; $i < $data['count']; $i++)
            $msg .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.t10_str'],
                                   $data[$i]['rank'],
                                   $data[$i]['score'],
                                   $data[$i]['player']);
         $msg .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.t10_player']);
         if ($data['player']['count'] == 0)
            $msg .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.t10_player_nf'],
                                   $view['playername']);
         else
            $msg .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.t10_str'],
                                   $data['player'][0]['rank'],
                                   $data['player'][0]['score'],
                                   $data['player'][0]['player']);
      }

      return $msg;
   }

   function msg_tt($view) {
      // data
      $data = $this->db->plug_tt($view);
      // format
      $this->cols_format('?', '', array(&$data, &$data['player']));
      // msg
      if ($data['count'] == 0)
         $msg  = $this->msg_str($view['st'], $this->cfg[$view['st'].'.tt_nf'],
                                $view['trackname']);
      else {
         $msg  = $this->msg_str($view['st'], $this->cfg[$view['st'].'.tt'],
                                $data[0]['track']);
         $msg .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.tt_author'],
                                $data[0]['authortime']);
         $msg .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.tt_gold'],
                                $data[0]['gold']);
         $msg .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.tt_silver'],
                                $data[0]['silver']);
         $msg .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.tt_bronze'],
                                $data[0]['bronze']);
         $msg .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.tt_player']);
         if ($data['player']['count'] == 0)
            $msg .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.tt_player_nf'],
                                   $view['playername']);
         else
            $msg .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.tt_player_str'],
                                   $data['player'][0]['result'],
                                   $data['player'][0]['player']);
      }

      return $msg;
   }

   function msg_recs($view) {
      // data
      $data = $this->db->plug_recs($view);
      // format
      $this->cols_format($view['st'], $this->cfg[$view['st'].'.recs_str'],
                         array(&$data, &$data['player']));
      // msg
      if ($data['count'] == 0)
         $msg = $this->msg_str($view['st'], $this->cfg[$view['st'].'.recs_nf']);
      else {
         $msg = $this->msg_str($view['st'], $this->cfg[$view['st'].'.recs'],
                               $view['trackname']);
         for ($i = 0; $i < $data['count']; $i++)
            $msg .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.recs_str'],
                                   $data[$i]['fp'],
                                   $data[$i]['result'],
                                   $data[$i]['player']);
         $msg .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.recs_player']);
         if ($data['player']['count'] == 0)
            $msg .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.recs_player_nf'],
                                   $view['playername']);
         else
            $msg .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.recs_str'],
                                   $data['player'][0]['fp'],
                                   $data['player'][0]['result'],
                                   $data['player'][0]['player']);
      }

      return $msg;
   }

   function msg_teams($view) {
      // data
      $data = $this->db->plug_teams($view);
      // format
      $this->cols_format($view['st'], $this->cfg[$view['st'].'.teams_str'],
                         array(&$data));
      // form msg
      if ($data['count'] == 0)
         $msg = $this->msg_str($view['st'],  $this->cfg[$view['st'].'.teams_nf']);
      else {
         $msg = $this->msg_str($view['st'],  $this->cfg[$view['st'].'.teams']);
         for ($i = 0; $i < $data['count']; $i++) {
            $msg .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.teams_str'],
                                   $data[$i]['rank'],
                                   $data[$i]['score'],
                                   $data[$i]['mc'],
                                   $data[$i]['clanc']);
         }
      }

      return $msg;
   }

   function msg_stats($view) {
      // data
      $data = $this->db->plug_stats($view);
      // format
      $this->cols_format('?', '', array(&$data));
      // msg
      if ($data['count'] == 0)
         $msg = $this->msg_str($view['st'], $this->cfg[$view['st'].'.stats_nf'],
                               $view['playername']);
      else {
         $msg  = $this->msg_str($view['st'], $this->cfg[$view['st'].'.stats'],
                                $data[0]['player']);
         $msg .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.stats_rank'], 
                                $data[0]['rank']);
         $msg .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.stats_score'],
                                $data[0]['score']);
         $msg .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.stats_medals'], 
                                $data[0]['am'], $data[0]['gm'],
                                $data[0]['sm'], $data[0]['bm']);
         $msg .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.stats_ft'], 
                                $data[0]['ft']);
         $msg .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.stats_fp'],
                                $data[0]['fp']);
         $msg .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.stats_afp'],
                                $data[0]['afp']);
      }

      return $msg;
   }

   function msg_srv($view) {
      // data
      $data = $this->db->plug_srv($view);
      // msg
      $msg  = $this->msg_str($view['st'], $this->cfg[$view['st'].'.srv'],
                             $data[0]['server']);
      $msg .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.srv_tp'],
                             $data[0]['tp']);
      $msg .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.srv_tt'],
                             $data[0]['tt']);
      $msg .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.srv_ta'],
                             $data[0]['tm']);
      $msg .= $this->msg_str($view['st'], $this->cfg[$view['st'].'.srv_tc'],
                             $data[0]['tc']);

      return $msg;
   }

   function msg_time($view) {
      $msg = $this->msg_str($view['st'], $this->cfg[$view['st'].'.time'],
                            ts_format(time()));

      return $msg;
   }

   function msg_unkcmd($view) {
      $msg = $this->msg_str($view['st'], $this->cfg[$view['st'].'.unknown']);

      return $msg;
   }

   function msg_title($view) {
      if (strpos("time,unk,off", $view['cmd']) !== false) {
         $msg = "\$z";
      }
      else if ($view['cmd'] == "help") {
         $msg = $this->cfg[$view['st'].".help"][0]['text']."\$z\n";
      }
      else {
         // data
         $data = $this->db->plug_lu($view);
         // format
         $this->cols_format('?', '', array(&$data));
         if ($view['cmd'] == "tt" ||
             $view['cmd'] == "recs")
            $envir = "";
         else if ($view['st'] == "dsm")
            $envir = substr($view['envir'], 0, 2).'.';
         else
            $envir = '#'.$view['envir'];
         // msg
         $msg = str_replace(array("{1}", "{2}", "{3}"),
                            array($view['cmd'], $data[0]['lastupdate'], $envir),
                            $this->cfg[$view['st'].".lu"][0]['text'])."\$z\n";
      }

      return $msg;
   }

   function msg_mlpheader($title) {
      $msg = "<manialink posx='{$this->cfg['mlp.posx']}' posy='{$this->cfg['mlp.posy']}' id='{$this->cfg['mlp.mlid']}'>".
             "<type>default</type>".
             "<format textsize=2></format>".
             "<background bgcolor='FFFD' bgborderx='0.005' bgbordery='0.005'></background>".
             "<line>".
             "<cell width='0.67' bgcolor='000F'><text halign='left'>\$fff".$title."</text></cell>".
             "<cell width='0.13' bgcolor='000F'><text halign='right' action='{$this->cfg['mlp.aid']}'>\$z\$fffclose  </text></cell>".
             "</line>";

      return $msg;
   }

   function msg_mlpfooter() {
      $msg = "</manialink>";

      return $msg;
   }

   function msg_mlpnull() {
      $msg = "<manialink id='{$this->cfg['mlp.mlid']}'/>";

      return $msg;
   }

   function msg_full($view) {
      // template
      $msg = array(
         "sendto"=>"",
         "title"=>"",
         "header"=>"",
         "body"=>"",
         "footer"=>"",
         "mlid"=>"",
         "aid"=>"",
         "posx"=>"",
         "posy"=>"",
         "width"=>"",
         "height"=>""
      );
      // null msg
      if ($view == null) {
         $msg['sendto'] = "mlp";
         $msg['body'] = $this->msg_mlpnull();
         $msg['mlid'] = $this->cfg['mlp.mlid'];
         $msg['aid'] = $this->cfg['mlp.aid'];
         return $msg;
      }
      // cmd
      $view['cmdtext'] = trim($view['cmdtext'], "\\/\t\n\r ");
      if (strpos($view['cmdtext'], 'tmos') === false)
         $view['cmdtext'] = "tmos ".$view['cmdtext'];
      $buf = preg_split("/\s/", $view['cmdtext'], 3, PREG_SPLIT_NO_EMPTY);
      foreach ($buf as $key=>$value)
         $buf[$key] = trim($value);
      if ($this->status['plug'] == "d")
         $view['cmd'] = "off";
      else if (!isset($buf[1]) || $buf[1] == "?" ||
               $buf[1] == "-?" || $buf[1] == '/?')
         $view['cmd'] = "help";
      else if (strpos("help,stats,top10,tt,recs,teams,srv,time",
                      $buf[1]) === false)
         $view['cmd'] = "unk";
      else
         $view['cmd'] = $buf[1];
      // envir
      if (!isset($buf[2]))
         $buf[2] = "Overall";
      else
         $buf[2] = ucfirst(strtolower($buf[2]));
      foreach(array('Overall', 'Stadium', 'Island', 'Bay',
                    'Coast', 'Alpine', 'Rally', 'Speed') as $envir) {
         if (substr($buf[2], 0, 2) == 
             substr($envir, 0, 2))
            $view['envir'] = $envir;
      }
      if (!isset($view['envir']))
         $view['envir'] = "Overall";
      // game
      $view['game'] = strtolower($view['game']);
      // sendto
      if ($this->cfg['write_to_chat'] ||
          strpos("off,unk,time", $view['cmd']) !== false ||
          strpos("tmo,tms", $view['game']) !== false)
         $view['st'] = "csm";
      else if ($view['game'] == "tmn")
         $view['st'] = "dsm";
      else
         $view['st'] = "mlp";
      // ids
      $view['sid'] = $this->cfg['dbserverid'];
      if (preg_match("/(.+)\/\d+\.\d+\.\d+\.\d+:\d+/u", $view['playeraccount'], $buf))
         $view['playeraccount'] = name_clear($buf[1]);
      $view['pid'] = id_get($view['playeraccount']);
      $view['tid'] = id_get($view['trackuid']);
      $view['eid'] = envir_toid($view['envir']);
      $view['pagenum'] = 1;
      $view['recperpage'] = 10;
      // full msg
      $msg['sendto'] = $view['st'];
      $msg['title'] = $this->msg_title($view);
      switch ($view['cmd']) {
         case "help"  : $msg['body'] = $this->msg_help($view); break;
         case "stats" : $msg['body'] = $this->msg_stats($view); break;
         case "top10" : $msg['body'] = $this->msg_top10($view); break;
         case "tt"    : $msg['body'] = $this->msg_tt($view); break;
         case "recs"  : $msg['body'] = $this->msg_recs($view); break;
         case "teams" : $msg['body'] = $this->msg_teams($view); break;
         case "srv"   : $msg['body'] = $this->msg_srv($view); break;
         case "time"  : $msg['body'] = $this->msg_time($view); break;
         case "unk"   : $msg['body'] = $this->msg_unkcmd($view); break;
         default      : $msg['body'] = ">> Disabled!"; break;
      }
      if ($view['st'] == "csm") {
         $msg['body'] = rtrim($msg['body']);
      }
      if ($view['st'] == "mlp") {
         $msg['title'] = rtrim($msg['title']);
         $msg['header'] = $this->msg_mlpheader($msg['title']);
         $msg['footer'] = $this->msg_mlpfooter();
         $msg['posx'] = $this->cfg['mlp.posx'];
         $msg['posy'] = $this->cfg['mlp.posy'];
         $msg['width'] = $this->cfg['mlp.width'];
         if (strpos("top10,recs,teams", $view['cmd']) !== false)
            $msg['height'] = $this->cfg['mlp.height2'];
         else
            $msg['height'] = $this->cfg['mlp.height1'];
      }

      return $msg;
   }
   
} // class

?>