<?php
/*
   TMOS plugin for (X)Aseco/FAST v5
   Copyright (c) 2006-2009 Alexander Domnin

   This program is licenced under the
   Creative Commons Attribution-Noncommercial-No Derivative Works 3.0 Unported.
   To view a copy of this licence, visit http://creativecommons.org/licenses/by-nc-nd/3.0/

   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
   without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*/

require_once("includes/tmos_plug.inc.php");
global $_FASTver;

////////////////////////////////////////////////////////////////////////////////
// aseco 0.x
// xaseco 0.x-1.x
// mistral aseco 0.x
if (class_exists('aseco') && 
    (!class_exists('plugin') ||
     defined('XASECO_VERSION') ||
     (defined('ASECO_VERSION') &&
      strpos(ASECO_VERSION, '0.') === 0))) {

   function tmosInit(&$aseco, &$command) {

      $aseco->_TMOS = new Ttmos_Plug();
      $result = $aseco->_TMOS->init("tmos.xml");
      for ($i = 0; $i < $result['count']; $i++)
         $aseco->console_text($result[$i]);

      // reopen aseco connection for localdb plugin, if needed
      global $ldb_settings;
      if (isset($ldb_settings)) {
         mysql_connect($ldb_settings['mysql']['host'], $ldb_settings['mysql']['login'],
                       $ldb_settings['mysql']['password'], true);
         mysql_select_db($ldb_settings['mysql']['database']);
      }
   }

   function chat_tmos(&$aseco, &$command) {
      $view = array (
         'cmdtext'=>$command['params'],
         'playername'=>$command['author']->nickname,
         'playeraccount'=>$command['author']->login,
         'trackname'=>$aseco->server->challenge->name,
         'trackuid'=>$aseco->server->challenge->uid,
         'game'=>$aseco->server->getGame()
      );

      $msg = $aseco->_TMOS->msg_full($view);
      if ($msg['sendto'] == "csm")
         $aseco->addCall('ChatSendServerMessageToLogin',
                         array($msg['title'].$msg['body'],
                               $command['author']->login));
      else if ($msg['sendto'] == "dsm")
         $aseco->addCall('SendDisplayServerMessageToLogin',
                         array($command['author']->login,
                               $msg['title'].$msg['body'], "OK", "", 0));
      else
         $aseco->addCall('SendDisplayManialinkPageToLogin',
                         array($command['author']->login,
                               $msg['header'].$msg['body'].$msg['footer'], 0, true));
   }

   function tmosPlayerManialinkPageAnswer(&$aseco, &$command) {
      $msg = $aseco->_TMOS->msg_full(null);
      if ($command[2] == $msg['aid']) {
         $aseco->addCall('SendDisplayManialinkPageToLogin',
                         array($command[1], $msg['body'], 0, false));
      }
   }

   Aseco::registerEvent("onStartup", "tmosInit");
   Aseco::addChatCommand("tmos", "displays TMOS in the game (for more: /tmos help)");
   Aseco::registerEvent("onPlayerManialinkPageAnswer", "tmosPlayerManialinkPageAnswer");
}

////////////////////////////////////////////////////////////////////////////////
// aseco tmu 1.x
// aseco tmf 2.x
else if (class_exists('aseco') && 
         class_exists('plugin')) {

   class TM_Offline_Stats extends Plugin {
      var $_TMOS;
      var $sc_ver;

      function tmosInit() {
         $this->_TMOS = new Ttmos_Plug();
         $result = $this->_TMOS->init("tmos.xml");
         for ($i = 0; $i < $result['count']; $i++)
            $this->Aseco->console_text($result[$i]);

         // reopen aseco connection for localdb plugin, if needed
         if (($aldb = $this->Aseco->getPlugin('LocalDb')) != null) {
            $aldb->connect();
         }
         
         if ((defined('ASECO_VERSION') &&
              strpos(ASECO_VERSION, '1.') === 0) ||
             !method_exists($this->Aseco, 'showManialinkFramework'))
            $this->sc_ver = 'U';
         else
            $this->sc_ver = 'F';
      }

      function chat_tmos($command) {
         $view = array (
            'cmdtext'=>$command['params'],
            'playername'=>$command['author']->nickname,
            'playeraccount'=>$command['author']->login,
            'trackname'=>$this->Aseco->server->challenge->name,
            'trackuid'=>$this->Aseco->server->challenge->uid,
            'game'=>$this->Aseco->server->getGame()
         );

         $msg = $this->_TMOS->msg_full($view);
         if ($msg['sendto'] == "csm") {
            $this->sendServerMessageToLogin($command['author']->login,
                                            $msg['title'].$msg['body']);
         }
         else if ($this->sc_ver == 'F') {
            $window = $this->createWindowToPlayerFramework($msg['title'],
               $msg['posx'], $msg['posy'], $command['author']);
            $window->setWidth($msg['width']);
            $window->setHeight($msg['height']);
            $window->setContent($msg['body']);
         }
         else {
            $window = $this->createWindowToPlayerFramework($msg['title'],
               $msg['posx'], $msg['posy'], $command['author']->login);
            $window->setWidth($msg['width']);
            $window->setContent($msg['body']);
         }
      }

   } // class

   $_PLUGIN = new TM_Offline_Stats();
   $_PLUGIN->setAuthor('Alexander Domnin');
   $_PLUGIN->setVersion(4);
   $_PLUGIN->addEvent('onStartup', 'tmosInit');
   $_PLUGIN->setWebsite('http://tmos.pp.ru');
   $_PLUGIN->addChatCommand('tmos', 'chat_tmos', 'displays TMOS in the game (for more: /tmos help)');
   $_PLUGIN->setDescription('This plugin lets you view statistics generated by "TM Offline Stats" directly in the game');
}

////////////////////////////////////////////////////////////////////////////////
// fast 3.x.x
else if (isset($_FASTver) &&
         strpos($_FASTver, '3.') === 0) {

   $_TMOS = NULL;

   function tmosInit($event) {
      global $_TMOS;

      $_TMOS = new Ttmos_Plug();
      $result = $_TMOS->init("tmos.xml");
      for ($i = 0; $i < $result['count']; $i++)
         console($result[$i]);
   }

   function chat_tmos($author, $login, $command) {
      global $_TMOS, $_Game, $_ChallengeInfo, $_PlayerInfo;

      $view = array (
         'cmdtext'=>'',
         'playername'=>$_PlayerInfo[$login]['NickName'],
         'playeraccount'=>$login,
         'trackname'=>$_ChallengeInfo['Name'],
         'trackuid'=>$_ChallengeInfo['UId'],
         'game'=>$_Game
      );
      foreach ($command as $part)
         $view['cmdtext'] .= $part.' ';

      $msg = $_TMOS->msg_full($view);
      if ($msg['sendto'] == "csm")
         addCall(null, "ChatSendServerMessageToLogin",
                 $msg['title'].$msg['body'], $login);
      else if ($msg['sendto'] == "dsm")
         addCall(null, "SendDisplayServerMessageToLogin",
                 $login, $msg['title'].$msg['body'], "OK", "", 0);
      else 
         addCall(null, 'SendDisplayManialinkPageToLogin',
                 $login, $msg['header'].$msg['body'].$msg['footer'], 0, false);
   }

   function tmosPlayerManialinkPageAnswer($event, $login, $answer) { //, $action) { <-- 3.2+
      global $_TMOS;

      $msg = $_TMOS->msg_full(null);
      if ($answer == $msg['aid']) {
         addCall(null, 'SendDisplayManialinkPageToLogin',
                 $login, $msg['body'], 0, false);
      }
   }

   registerPlugin('tmos', 50, 4.0);
   registerCommand('tmos','/tmos : displays TMOS in the game (for more: /tmos help)');
}

////////////////////////////////////////////////////////////////////////////////
// ???
else {
   die ('TMOS - WTF?');
}

?>